.TH RNAHEAT l
.ER
.SH NAME
RNAheat \- calculate specific heat of RNAs
.SH SYNOPSIS
\fBRNAheat [\-Tmin \fIt1\fP] [\-Tmax \fIt2\fP] [\-h \fIstepsize\fP] [\-m \fIipoints\fP]
        [\-4] [\-noGU] [-e \fI1|2\fP]
.SH DESCRIPTION
.I RNAheat
reads RNA sequences from stdin and calculates their specific heat
in the temperature range \fIt1\fP to \fIt2\fP, from the partition
function by numeric differentiation. The result is written to stdout
as a list of pairs of temperature in C and specific heat in Kcal/(Mol*K).
.br
The program will continue to read new sequences until a line consisting
of the single character @ or an end of file condition is encountered.
.SH OPTIONS
.IP \fB\-Tmin\ \fIt1\fP
Lowest temperature, default is 0C.
.IP \fB\-Tmax\ \fIt2\fP
Highest temperature, default is 100C.
.IP \fB\-h\ \fIstepsize\fP
Calculate partition function every \fIstepsize\fP degrees C. Default
is 1C.
.IP \fB\-m\ \fIipoints\fP
The program fits a parabola to 2*ipoints+1 data points to calculate
2nd derivatives. Increasing this parameter produces a smoother curve.
Default is 2.
.IP \fB\-4\fB
Do not include special stabilizing energies for certain tetraloops.
.IP \fB\-noGU\fB
Do not allow GU pairs.
.IP \fB\-e\ \fI1|2\fP\fB
Fold sequences from the artificial ABCD... alphabet, where A pairs B,
C-D etc.  Use the energy parameters for GC (-e 1) or AU (-e 2) pairs.
.SH EXAMPLES
.nf
	RNAheat -Tmin -30 -Tmax 120 -h0.1 -m4 < tRNA.seq | xvgr -pipe
.nf
produces plots of the specific heat of the sequences in tRNA.seq.
.SH REFERENCES
The partition function algorithm is based on work by J.S. McCaskill.
The energy parameters are taken from:
.br
Freier et al Proc.Nat.Acad.Sci (1986) 83, p. 9373
.br
Jaeger et al Proc.Nat.Acad.Sci (1989) 86, p. 7706
.br
He et al Biochemistry (1991) 30, p. 11124
.PP
If you use this program in your work you might want to cite:
.PP
I.L. Hofacker, W. Fontana, P.F. Stadler, S. Bonherffer, M. Tacker, P. Schuster 
(1994)
Fast Folding and Comparison of RNA Secondary Structures.
Monatshefte f. Chemie 125: 167-188
.PP
J.S. McCaskill (1990) The equilibrium partition function and base pair
binding probabilities for RNA secondary structures, Biopolymers 29: 1105-1119
.br
D.H. Turner N. Sugimoto and S.M. Freier (1988) RNA structure
prediction, Ann Rev Biophys Biophys Chem 17: 167-192
.br
D. Adams (1979) The hitchhiker's guide to the galaxy, Pan Books, London
.SH VERSION
This man page documents RNAheat 1.00.42  09/21/94
.SH AUTHORS
Ivo L Hofacker, Peter F Stadler.
.SH SEE ALSO
.BR RNAfold (l),
.BR RNAdistance (l)
.SH BUGS
If in doubt our program is right nature is at fault.
Comments should be sent to ivo@tbi.univie.ac.at.
.br
